from tracemalloc import take_snapshot
import numpy as np
from .ProblemData import ProblemData
import pinocchio as pin


class Target:
    """Targets of the whole-body tasks

    Args:
        params (Params): object storing parameters about the control problem as a whole

    Attributes:
        params (Params): object storing parameters about the control problem as a whole
        dt_wbc (float): time step of the low level integration
        linear_velocity (array): velocity target for the base linear velocity tracking task
        angular_velocity (array): velocity target for the base angular velocity tracking task
        velocity_task (list of arrays): targets for linear and angular velocities
    """

    def __init__(self, params):
        self.params = params
        self.dt_wbc = params.dt_wbc

        self.linear_velocity = np.array([0.1, 0, 0])
        self.angular_velocity = np.array([0, 0, 0])
        self.velocity_task = [self.linear_velocity, self.angular_velocity]
